@@ -91,8 +91,6 @@ bool Save(const std::string& filename,
           const bool fatal = false,
           format f = format::autodetect);
 
-#ifdef HAS_STB
-
 /**
  * Save the image file from the given matrix.
  *
@@ -107,8 +105,7 @@ template<typename eT>
 bool Save(const std::string& filename,
           arma::Mat<eT>& matrix,
           ImageInfo& info,
-          const bool fatal = false,
-          const bool transpose = true);
+          const bool fatal = false);
 
 /**
  * Save the image file from the given matrix.
@@ -124,10 +121,15 @@ template<typename eT>
 bool Save(const std::vector<std::string>& files,
           arma::Mat<eT>& matrix,
           ImageInfo& info,
-          const bool fatal = false,
-          const bool transpose = true);
+          const bool fatal = false);
 
-#endif // HAS_STB.
+/**
+ * Helper function to save files.  Implementation in save_image.cpp.
+ */
+bool SaveImage(const std::string& filename,
+               arma::Mat<unsigned char>& image,
+               ImageInfo& info,
+               const bool fatal = false);
 
 } // namespace data
 } // namespace mlpack
