@@ -41,7 +41,7 @@ FFN<OutputLayerType, InitializationRuleType, CustomLayers...>::FFN(
     numFunctions(0),
     deterministic(true)
 {
-  /* Nothing to do here */
+  /* Nothing to do here. */
 }
 
 template<typename OutputLayerType, typename InitializationRuleType,
@@ -67,6 +67,41 @@ void FFN<OutputLayerType, InitializationRuleType, CustomLayers...>::ResetData(
     ResetParameters();
 }
 
+template<typename OutputLayerType, typename InitializationRuleType,
+         typename... CustomLayers>
+template<typename OptimizerType>
+typename std::enable_if<
+      HasMaxIterations<OptimizerType, size_t&(OptimizerType::*)()>
+      ::value, void>::type
+FFN<OutputLayerType, InitializationRuleType, CustomLayers...>::
+WarnMessageMaxIterations(OptimizerType& optimizer, size_t samples) const
+{
+  if (optimizer.MaxIterations() < samples &&
+      optimizer.MaxIterations() != 0)
+  {
+    Log::Warn << "The optimizer's maximum number of iterations "
+              << "is less than the size of the dataset; the "
+              << "optimizer will not pass over the entire "
+              << "dataset. To fix this, modify the maximum "
+              << "number of iterations to be at least equal "
+              << "to the number of points of your dataset "
+              << "(" << samples << ")." << std::endl;
+  }
+}
+
+template<typename OutputLayerType, typename InitializationRuleType,
+         typename... CustomLayers>
+template<typename OptimizerType>
+typename std::enable_if<
+      !HasMaxIterations<OptimizerType, size_t&(OptimizerType::*)()>
+      ::value, void>::type
+FFN<OutputLayerType, InitializationRuleType, CustomLayers...>::
+WarnMessageMaxIterations(OptimizerType& /* optimizer */, size_t /* samples */)
+    const
+{
+  return;
+}
+
 template<typename OutputLayerType, typename InitializationRuleType,
          typename... CustomLayers>
 template<typename OptimizerType, typename... CallbackTypes>
@@ -78,6 +113,8 @@ double FFN<OutputLayerType, InitializationRuleType, CustomLayers...>::Train(
 {
   ResetData(std::move(predictors), std::move(responses));
 
+  WarnMessageMaxIterations<OptimizerType>(optimizer, this->predictors.n_cols);
+
   // Train the model.
   Timer::Start("ffn_optimization");
   const double out = optimizer.Optimize(*this, parameter, callbacks...);
@@ -100,6 +137,8 @@ double FFN<OutputLayerType, InitializationRuleType, CustomLayers...>::Train(
 
   OptimizerType optimizer;
 
+  WarnMessageMaxIterations<OptimizerType>(optimizer, this->predictors.n_cols);
+
   // Train the model.
   Timer::Start("ffn_optimization");
   const double out = optimizer.Optimize(*this, parameter, callbacks...);
@@ -112,8 +151,9 @@ double FFN<OutputLayerType, InitializationRuleType, CustomLayers...>::Train(
 
 template<typename OutputLayerType, typename InitializationRuleType,
          typename... CustomLayers>
+template<typename PredictorsType, typename ResponsesType>
 void FFN<OutputLayerType, InitializationRuleType, CustomLayers...>::Forward(
-    arma::mat inputs, arma::mat& results)
+    const PredictorsType& inputs, ResponsesType& results)
 {
   if (parameter.is_empty())
     ResetParameters();
@@ -124,25 +164,28 @@ void FFN<OutputLayerType, InitializationRuleType, CustomLayers...>::Forward(
     ResetDeterministic();
   }
 
-  currentInput = std::move(inputs);
-  Forward(std::move(currentInput));
+  Forward(inputs);
   results = boost::apply_visitor(outputParameterVisitor, network.back());
 }
 
 template<typename OutputLayerType, typename InitializationRuleType,
          typename... CustomLayers>
+template<typename PredictorsType, typename ResponsesType>
 void FFN<OutputLayerType, InitializationRuleType, CustomLayers...>::Forward(
-    arma::mat inputs, arma::mat& results, const size_t begin, const size_t end)
+    const PredictorsType& inputs,
+    ResponsesType& results,
+    const size_t begin,
+    const size_t end)
 {
-  boost::apply_visitor(ForwardVisitor(std::move(inputs), std::move(
-      boost::apply_visitor(outputParameterVisitor, network[begin]))),
+  boost::apply_visitor(ForwardVisitor(inputs,
+      boost::apply_visitor(outputParameterVisitor, network[begin])),
       network[begin]);
 
   for (size_t i = 1; i < end - begin + 1; ++i)
   {
-    boost::apply_visitor(ForwardVisitor(std::move(boost::apply_visitor(
-        outputParameterVisitor, network[begin + i - 1])), std::move(
-        boost::apply_visitor(outputParameterVisitor, network[begin + i]))),
+    boost::apply_visitor(ForwardVisitor(boost::apply_visitor(
+        outputParameterVisitor, network[begin + i - 1]),
+        boost::apply_visitor(outputParameterVisitor, network[begin + i])),
         network[begin + i]);
   }
 
@@ -151,25 +194,28 @@ void FFN<OutputLayerType, InitializationRuleType, CustomLayers...>::Forward(
 
 template<typename OutputLayerType, typename InitializationRuleType,
          typename... CustomLayers>
+template<typename PredictorsType, typename TargetsType, typename GradientsType>
 double FFN<OutputLayerType, InitializationRuleType, CustomLayers...>::Backward(
-    arma::mat targets, arma::mat& gradients)
+    const PredictorsType& inputs,
+    const TargetsType& targets,
+    GradientsType& gradients)
 {
-  double res = outputLayer.Forward(std::move(boost::apply_visitor(
-      outputParameterVisitor, network.back())), std::move(targets));
+  double res = outputLayer.Forward(boost::apply_visitor(
+      outputParameterVisitor, network.back()), targets);
 
   for (size_t i = 0; i < network.size(); ++i)
   {
     res += boost::apply_visitor(lossVisitor, network[i]);
   }
 
-  outputLayer.Backward(std::move(boost::apply_visitor(outputParameterVisitor,
-      network.back())), std::move(targets), std::move(error));
+  outputLayer.Backward(boost::apply_visitor(outputParameterVisitor,
+      network.back()), targets, error);
 
   gradients = arma::zeros<arma::mat>(parameter.n_rows, parameter.n_cols);
 
   Backward();
   ResetGradients(gradients);
-  Gradient(std::move(currentInput));
+  Gradient(inputs);
 
   return res;
 }
@@ -189,8 +235,7 @@ void FFN<OutputLayerType, InitializationRuleType, CustomLayers...>::Predict(
   }
 
   arma::mat resultsTemp;
-  Forward(std::move(arma::mat(predictors.colptr(0),
-      predictors.n_rows, 1, false, true)));
+  Forward(arma::mat(predictors.colptr(0), predictors.n_rows, 1, false, true));
   resultsTemp = boost::apply_visitor(outputParameterVisitor,
       network.back()).col(0);
 
@@ -199,8 +244,7 @@ void FFN<OutputLayerType, InitializationRuleType, CustomLayers...>::Predict(
 
   for (size_t i = 1; i < predictors.n_cols; i++)
   {
-    Forward(std::move(arma::mat(predictors.colptr(i),
-        predictors.n_rows, 1, false, true)));
+    Forward(arma::mat(predictors.colptr(i), predictors.n_rows, 1, false, true));
 
     resultsTemp = boost::apply_visitor(outputParameterVisitor,
         network.back());
@@ -210,8 +254,9 @@ void FFN<OutputLayerType, InitializationRuleType, CustomLayers...>::Predict(
 
 template<typename OutputLayerType, typename InitializationRuleType,
          typename... CustomLayers>
+template<typename PredictorsType, typename ResponsesType>
 double FFN<OutputLayerType, InitializationRuleType, CustomLayers...>::Evaluate(
-    arma::mat predictors, arma::mat responses)
+    const PredictorsType& predictors, const ResponsesType& responses)
 {
   if (parameter.is_empty())
     ResetParameters();
@@ -222,10 +267,10 @@ double FFN<OutputLayerType, InitializationRuleType, CustomLayers...>::Evaluate(
     ResetDeterministic();
   }
 
-  Forward(std::move(predictors));
+  Forward(predictors);
 
-  double res = outputLayer.Forward(std::move(boost::apply_visitor(
-      outputParameterVisitor, network.back())), std::move(responses));
+  double res = outputLayer.Forward(boost::apply_visitor(
+      outputParameterVisitor, network.back()), responses);
 
   for (size_t i = 0; i < network.size(); ++i)
   {
@@ -264,10 +309,10 @@ double FFN<OutputLayerType, InitializationRuleType, CustomLayers...>::Evaluate(
     ResetDeterministic();
   }
 
-  Forward(std::move(predictors.cols(begin, begin + batchSize - 1)));
+  Forward(predictors.cols(begin, begin + batchSize - 1));
   double res = outputLayer.Forward(
-      std::move(boost::apply_visitor(outputParameterVisitor, network.back())),
-      std::move(responses.cols(begin, begin + batchSize - 1)));
+      boost::apply_visitor(outputParameterVisitor, network.back()),
+      responses.cols(begin, begin + batchSize - 1));
 
   for (size_t i = 0; i < network.size(); ++i)
   {
@@ -325,10 +370,10 @@ EvaluateWithGradient(const arma::mat& /* parameters */,
     ResetDeterministic();
   }
 
-  Forward(std::move(predictors.cols(begin, begin + batchSize - 1)));
+  Forward(predictors.cols(begin, begin + batchSize - 1));
   double res = outputLayer.Forward(
-      std::move(boost::apply_visitor(outputParameterVisitor, network.back())),
-      std::move(responses.cols(begin, begin + batchSize - 1)));
+      boost::apply_visitor(outputParameterVisitor, network.back()),
+      responses.cols(begin, begin + batchSize - 1));
 
   for (size_t i = 0; i < network.size(); ++i)
   {
@@ -336,13 +381,13 @@ EvaluateWithGradient(const arma::mat& /* parameters */,
   }
 
   outputLayer.Backward(
-      std::move(boost::apply_visitor(outputParameterVisitor, network.back())),
-      std::move(responses.cols(begin, begin + batchSize - 1)),
-      std::move(error));
+      boost::apply_visitor(outputParameterVisitor, network.back()),
+      responses.cols(begin, begin + batchSize - 1),
+      error);
 
   Backward();
   ResetGradients(gradient);
-  Gradient(std::move(predictors.cols(begin, begin + batchSize - 1)));
+  Gradient(predictors.cols(begin, begin + batchSize - 1));
 
   return res;
 }
@@ -396,18 +441,19 @@ void FFN<OutputLayerType, InitializationRuleType,
   size_t offset = 0;
   for (size_t i = 0; i < network.size(); ++i)
   {
-    offset += boost::apply_visitor(GradientSetVisitor(std::move(gradient),
-        offset), network[i]);
+    offset += boost::apply_visitor(GradientSetVisitor(gradient, offset),
+        network[i]);
   }
 }
 
 template<typename OutputLayerType, typename InitializationRuleType,
          typename... CustomLayers>
+template<typename InputType>
 void FFN<OutputLayerType, InitializationRuleType,
-         CustomLayers...>::Forward(arma::mat&& input)
+         CustomLayers...>::Forward(const InputType& input)
 {
-  boost::apply_visitor(ForwardVisitor(std::move(input), std::move(
-      boost::apply_visitor(outputParameterVisitor, network.front()))),
+  boost::apply_visitor(ForwardVisitor(input,
+      boost::apply_visitor(outputParameterVisitor, network.front())),
       network.front());
 
   if (!reset)
@@ -434,9 +480,9 @@ void FFN<OutputLayerType, InitializationRuleType,
       boost::apply_visitor(SetInputHeightVisitor(height), network[i]);
     }
 
-    boost::apply_visitor(ForwardVisitor(std::move(boost::apply_visitor(
-        outputParameterVisitor, network[i - 1])), std::move(
-        boost::apply_visitor(outputParameterVisitor, network[i]))), network[i]);
+    boost::apply_visitor(ForwardVisitor(boost::apply_visitor(
+        outputParameterVisitor, network[i - 1]),
+        boost::apply_visitor(outputParameterVisitor, network[i])), network[i]);
 
     if (!reset)
     {
@@ -462,37 +508,38 @@ template<typename OutputLayerType, typename InitializationRuleType,
          typename... CustomLayers>
 void FFN<OutputLayerType, InitializationRuleType, CustomLayers...>::Backward()
 {
-  boost::apply_visitor(BackwardVisitor(std::move(boost::apply_visitor(
-      outputParameterVisitor, network.back())), std::move(error), std::move(
-      boost::apply_visitor(deltaVisitor, network.back()))), network.back());
+  boost::apply_visitor(BackwardVisitor(boost::apply_visitor(
+      outputParameterVisitor, network.back()), error,
+      boost::apply_visitor(deltaVisitor, network.back())), network.back());
 
   for (size_t i = 2; i < network.size(); ++i)
   {
-    boost::apply_visitor(BackwardVisitor(std::move(boost::apply_visitor(
-        outputParameterVisitor, network[network.size() - i])), std::move(
-        boost::apply_visitor(deltaVisitor, network[network.size() - i + 1])),
-        std::move(boost::apply_visitor(deltaVisitor,
-        network[network.size() - i]))), network[network.size() - i]);
+    boost::apply_visitor(BackwardVisitor(boost::apply_visitor(
+        outputParameterVisitor, network[network.size() - i]),
+        boost::apply_visitor(deltaVisitor, network[network.size() - i + 1]),
+        boost::apply_visitor(deltaVisitor, network[network.size() - i])),
+        network[network.size() - i]);
   }
 }
 
 template<typename OutputLayerType, typename InitializationRuleType,
          typename... CustomLayers>
+template<typename InputType>
 void FFN<OutputLayerType, InitializationRuleType,
-         CustomLayers...>::Gradient(arma::mat&& input)
+         CustomLayers...>::Gradient(const InputType& input)
 {
-  boost::apply_visitor(GradientVisitor(std::move(input), std::move(
-      boost::apply_visitor(deltaVisitor, network[1]))), network.front());
+  boost::apply_visitor(GradientVisitor(input,
+      boost::apply_visitor(deltaVisitor, network[1])), network.front());
 
   for (size_t i = 1; i < network.size() - 1; ++i)
   {
-    boost::apply_visitor(GradientVisitor(std::move(boost::apply_visitor(
-        outputParameterVisitor, network[i - 1])), std::move(
-        boost::apply_visitor(deltaVisitor, network[i + 1]))), network[i]);
+    boost::apply_visitor(GradientVisitor(boost::apply_visitor(
+        outputParameterVisitor, network[i - 1]),
+        boost::apply_visitor(deltaVisitor, network[i + 1])), network[i]);
   }
 
-  boost::apply_visitor(GradientVisitor(std::move(boost::apply_visitor(
-      outputParameterVisitor, network[network.size() - 2])), std::move(error)),
+  boost::apply_visitor(GradientVisitor(boost::apply_visitor(
+      outputParameterVisitor, network[network.size() - 2]), error),
       network[network.size() - 1]);
 }
 
@@ -505,7 +552,13 @@ void FFN<OutputLayerType, InitializationRuleType, CustomLayers...>::serialize(
   ar & BOOST_SERIALIZATION_NVP(parameter);
   ar & BOOST_SERIALIZATION_NVP(width);
   ar & BOOST_SERIALIZATION_NVP(height);
-  ar & BOOST_SERIALIZATION_NVP(currentInput);
+
+  // Early versions used the currentInput member, which is now no longer needed.
+  if (version < 2)
+  {
+    arma::mat currentInput; // Temporary matrix to output.
+    ar & BOOST_SERIALIZATION_NVP(currentInput);
+  }
 
   // Earlier versions of the FFN code did not serialize whether or not the model
   // was reset.
@@ -535,8 +588,8 @@ void FFN<OutputLayerType, InitializationRuleType, CustomLayers...>::serialize(
     size_t offset = 0;
     for (size_t i = 0; i < network.size(); ++i)
     {
-      offset += boost::apply_visitor(WeightSetVisitor(std::move(parameter),
-          offset), network[i]);
+      offset += boost::apply_visitor(WeightSetVisitor(parameter, offset),
+          network[i]);
 
       boost::apply_visitor(resetVisitor, network[i]);
     }
@@ -562,7 +615,6 @@ void FFN<OutputLayerType, InitializationRuleType,
   std::swap(parameter, network.parameter);
   std::swap(numFunctions, network.numFunctions);
   std::swap(error, network.error);
-  std::swap(currentInput, network.currentInput);
   std::swap(deterministic, network.deterministic);
   std::swap(delta, network.delta);
   std::swap(inputParameter, network.inputParameter);
@@ -584,7 +636,6 @@ FFN<OutputLayerType, InitializationRuleType, CustomLayers...>::FFN(
     parameter(network.parameter),
     numFunctions(network.numFunctions),
     error(network.error),
-    currentInput(network.currentInput),
     deterministic(network.deterministic),
     delta(network.delta),
     inputParameter(network.inputParameter),
@@ -613,7 +664,6 @@ FFN<OutputLayerType, InitializationRuleType, CustomLayers...>::FFN(
     parameter(std::move(network.parameter)),
     numFunctions(network.numFunctions),
     error(std::move(network.error)),
-    currentInput(std::move(network.currentInput)),
     deterministic(network.deterministic),
     delta(std::move(network.delta)),
     inputParameter(std::move(network.inputParameter)),
