@@ -51,7 +51,7 @@ class Constant
    * @param output Resulting output activation.
    */
   template<typename InputType, typename OutputType>
-  void Forward(const InputType&& input, OutputType&& output);
+  void Forward(const InputType& input, OutputType& output);
 
   /**
    * Ordinary feed backward pass of a neural network. The backward pass of the
@@ -62,9 +62,9 @@ class Constant
    * @param g The calculated gradient.
    */
   template<typename DataType>
-  void Backward(const DataType&& /* input */,
-                DataType&& /* gy */,
-                DataType&& g);
+  void Backward(const DataType& /* input */,
+                const DataType& /* gy */,
+                DataType& g);
 
   //! Get the output parameter.
   OutputDataType& OutputParameter() const { return outputParameter; }
