@@ -52,50 +52,29 @@ Convolution<
 >::Convolution(
     const size_t inSize,
     const size_t outSize,
-    const size_t kW,
-    const size_t kH,
-    const size_t dW,
-    const size_t dH,
+    const size_t kernelWidth,
+    const size_t kernelHeight,
+    const size_t strideWidth,
+    const size_t strideHeight,
     const size_t padW,
     const size_t padH,
     const size_t inputWidth,
     const size_t inputHeight,
-    const std::string paddingType) :
-    inSize(inSize),
-    outSize(outSize),
-    kW(kW),
-    kH(kH),
-    dW(dW),
-    dH(dH),
-    padWLeft(padW),
-    padWRight(padW),
-    padHBottom(padH),
-    padHTop(padH),
-    inputWidth(inputWidth),
-    inputHeight(inputHeight),
-    outputWidth(0),
-    outputHeight(0)
+    const std::string& paddingType) :
+    Convolution(
+      inSize,
+      outSize,
+      kernelWidth,
+      kernelHeight,
+      strideWidth,
+      strideHeight,
+      std::tuple<size_t, size_t>(padW, padW),
+      std::tuple<size_t, size_t>(padH, padH),
+      inputWidth,
+      inputHeight,
+      paddingType)
 {
-  weights.set_size((outSize * inSize * kW * kH) + outSize, 1);
-
-  // Transform paddingType to lowercase.
-  std::string paddingTypeLow = paddingType;
-  std::transform(paddingType.begin(), paddingType.end(), paddingTypeLow.begin(),
-      [](unsigned char c){ return std::tolower(c); });
-
-  if (paddingTypeLow == "valid")
-  {
-    padWLeft = 0;
-    padWRight = 0;
-    padHTop = 0;
-    padHBottom = 0;
-  }
-  else if (paddingTypeLow == "same")
-  {
-    InitializeSamePadding();
-  }
-
-  padding = new Padding<>(padWLeft, padWRight, padHTop, padHBottom);
+  // Nothing to do here.
 }
 
 template<
@@ -114,21 +93,21 @@ Convolution<
 >::Convolution(
     const size_t inSize,
     const size_t outSize,
-    const size_t kW,
-    const size_t kH,
-    const size_t dW,
-    const size_t dH,
-    const std::tuple<size_t, size_t> padW,
-    const std::tuple<size_t, size_t> padH,
+    const size_t kernelWidth,
+    const size_t kernelHeight,
+    const size_t strideWidth,
+    const size_t strideHeight,
+    const std::tuple<size_t, size_t>& padW,
+    const std::tuple<size_t, size_t>& padH,
     const size_t inputWidth,
     const size_t inputHeight,
-    const std::string paddingType) :
+    const std::string& paddingType) :
     inSize(inSize),
     outSize(outSize),
-    kW(kW),
-    kH(kH),
-    dW(dW),
-    dH(dH),
+    kernelWidth(kernelWidth),
+    kernelHeight(kernelHeight),
+    strideWidth(strideWidth),
+    strideHeight(strideHeight),
     padWLeft(std::get<0>(padW)),
     padWRight(std::get<1>(padW)),
     padHBottom(std::get<1>(padH)),
@@ -138,7 +117,8 @@ Convolution<
     outputWidth(0),
     outputHeight(0)
 {
-  weights.set_size((outSize * inSize * kW * kH) + outSize, 1);
+  weights.set_size((outSize * inSize * kernelWidth * kernelHeight) + outSize,
+      1);
 
   // Transform paddingType to lowercase.
   std::string paddingTypeLow = paddingType;
@@ -157,7 +137,7 @@ Convolution<
     InitializeSamePadding();
   }
 
-  padding = new Padding<>(padWLeft, padWRight, padHTop, padHBottom);
+  padding = ann::Padding<>(padWLeft, padWRight, padHTop, padHBottom);
 }
 
 template<
@@ -175,7 +155,7 @@ void Convolution<
     OutputDataType
 >::Reset()
 {
-    weight = arma::cube(weights.memptr(), kW, kH,
+    weight = arma::cube(weights.memptr(), kernelWidth, kernelHeight,
         outSize * inSize, false, false);
     bias = arma::mat(weights.memptr() + weight.n_elem,
         outSize, 1, false, false);
@@ -195,10 +175,10 @@ void Convolution<
     GradientConvolutionRule,
     InputDataType,
     OutputDataType
->::Forward(const arma::Mat<eT>&& input, arma::Mat<eT>&& output)
+>::Forward(const arma::Mat<eT>& input, arma::Mat<eT>& output)
 {
   batchSize = input.n_cols;
-  inputTemp = arma::cube(const_cast<arma::Mat<eT>&&>(input).memptr(),
+  arma::cube inputTemp(const_cast<arma::Mat<eT>&>(input).memptr(),
       inputWidth, inputHeight, inSize * batchSize, false, false);
 
   if (padWLeft != 0 || padWRight != 0 || padHTop != 0 || padHBottom != 0)
@@ -208,13 +188,14 @@ void Convolution<
 
     for (size_t i = 0; i < inputTemp.n_slices; ++i)
     {
-      padding->Forward(std::move(inputTemp.slice(i)),
-          std::move(inputPaddedTemp.slice(i)));
+      padding.Forward(inputTemp.slice(i), inputPaddedTemp.slice(i));
     }
   }
 
-  size_t wConv = ConvOutSize(inputWidth, kW, dW, padWLeft, padWRight);
-  size_t hConv = ConvOutSize(inputHeight, kH, dH, padHTop, padHBottom);
+  size_t wConv = ConvOutSize(inputWidth, kernelWidth, strideWidth, padWLeft,
+      padWRight);
+  size_t hConv = ConvOutSize(inputHeight, kernelHeight, strideHeight, padHTop,
+      padHBottom);
 
   output.set_size(wConv * hConv * outSize, batchSize);
   outputTemp = arma::Cube<eT>(output.memptr(), wConv, hConv,
@@ -237,12 +218,14 @@ void Convolution<
       if (padWLeft != 0 || padWRight != 0 || padHTop != 0 || padHBottom != 0)
       {
         ForwardConvolutionRule::Convolution(inputPaddedTemp.slice(inMap +
-            batchCount * inSize), weight.slice(outMapIdx), convOutput, dW, dH);
+            batchCount * inSize), weight.slice(outMapIdx), convOutput,
+            strideWidth, strideHeight);
       }
       else
       {
         ForwardConvolutionRule::Convolution(inputTemp.slice(inMap +
-            batchCount * inSize), weight.slice(outMapIdx), convOutput, dW, dH);
+            batchCount * inSize), weight.slice(outMapIdx), convOutput,
+            strideWidth, strideHeight);
       }
 
       outputTemp.slice(outMap) += convOutput;
@@ -270,14 +253,14 @@ void Convolution<
     InputDataType,
     OutputDataType
 >::Backward(
-    const arma::Mat<eT>&& /* input */, arma::Mat<eT>&& gy, arma::Mat<eT>&& g)
+    const arma::Mat<eT>& /* input */, const arma::Mat<eT>& gy, arma::Mat<eT>& g)
 {
-  arma::cube mappedError(gy.memptr(), outputWidth, outputHeight,
-      outSize * batchSize, false, false);
+  arma::cube mappedError(((arma::Mat<eT>&) gy).memptr(), outputWidth,
+      outputHeight, outSize * batchSize, false, false);
 
-  g.set_size(inputTemp.n_rows * inputTemp.n_cols * inSize, batchSize);
-  gTemp = arma::Cube<eT>(g.memptr(), inputTemp.n_rows,
-      inputTemp.n_cols, inputTemp.n_slices, false, false);
+  g.set_size(inputWidth * inputHeight * inSize, batchSize);
+  gTemp = arma::Cube<eT>(g.memptr(), inputWidth, inputHeight,
+      inSize * batchSize, false, false);
   gTemp.zeros();
 
   for (size_t outMap = 0, outMapIdx = 0, batchCount = 0; outMap <
@@ -295,7 +278,7 @@ void Convolution<
       Rotate180(weight.slice(outMapIdx), rotatedFilter);
 
       BackwardConvolutionRule::Convolution(mappedError.slice(outMap),
-          rotatedFilter, output, dW, dH);
+          rotatedFilter, output, strideWidth, strideHeight);
 
       if (padWLeft != 0 || padWRight != 0 || padHTop != 0 || padHBottom != 0)
       {
@@ -325,12 +308,14 @@ void Convolution<
     InputDataType,
     OutputDataType
 >::Gradient(
-    const arma::Mat<eT>&& /* input */,
-    arma::Mat<eT>&& error,
-    arma::Mat<eT>&& gradient)
+    const arma::Mat<eT>& input,
+    const arma::Mat<eT>& error,
+    arma::Mat<eT>& gradient)
 {
-  arma::cube mappedError(error.memptr(), outputWidth,
+  arma::cube mappedError(((arma::Mat<eT>&) error).memptr(), outputWidth,
       outputHeight, outSize * batchSize, false, false);
+  arma::cube inputTemp(((arma::Mat<eT>&) input).memptr(), inputWidth,
+      inputHeight, inSize * batchSize, false, false);
 
   gradient.set_size(weights.n_elem, 1);
   gradientTemp = arma::Cube<eT>(gradient.memptr(), weight.n_rows,
@@ -362,7 +347,7 @@ void Convolution<
 
       arma::Mat<eT> output;
       GradientConvolutionRule::Convolution(inputSlice, deltaSlice,
-          output, dW, dH);
+          output, strideWidth, strideHeight);
 
       if (gradientTemp.n_rows < output.n_rows ||
           gradientTemp.n_cols < output.n_cols)
@@ -406,10 +391,10 @@ void Convolution<
   ar & BOOST_SERIALIZATION_NVP(inSize);
   ar & BOOST_SERIALIZATION_NVP(outSize);
   ar & BOOST_SERIALIZATION_NVP(batchSize);
-  ar & BOOST_SERIALIZATION_NVP(kW);
-  ar & BOOST_SERIALIZATION_NVP(kH);
-  ar & BOOST_SERIALIZATION_NVP(dW);
-  ar & BOOST_SERIALIZATION_NVP(dH);
+  ar & BOOST_SERIALIZATION_NVP(kernelWidth);
+  ar & BOOST_SERIALIZATION_NVP(kernelHeight);
+  ar & BOOST_SERIALIZATION_NVP(strideWidth);
+  ar & BOOST_SERIALIZATION_NVP(strideHeight);
   ar & BOOST_SERIALIZATION_NVP(padWLeft);
   ar & BOOST_SERIALIZATION_NVP(padWRight);
   ar & BOOST_SERIALIZATION_NVP(padHBottom);
@@ -423,7 +408,10 @@ void Convolution<
     ar & BOOST_SERIALIZATION_NVP(padding);
 
   if (Archive::is_loading::value)
-    weights.set_size((outSize * inSize * kW * kH) + outSize, 1);
+  {
+    weights.set_size((outSize * inSize * kernelWidth * kernelHeight) + outSize,
+        1);
+  }
 }
 
 template<
@@ -444,8 +432,10 @@ void Convolution<
   /*
    * Using O = (W - F + 2P) / s + 1;
    */
-  size_t totalVerticalPadding = (dW - 1) * inputWidth + kW - dW;
-  size_t totalHorizontalPadding = (dH - 1) * inputHeight + kH - dH;
+  size_t totalVerticalPadding = (strideWidth - 1) * inputWidth + kernelWidth -
+      strideWidth;
+  size_t totalHorizontalPadding = (strideHeight - 1) * inputHeight +
+      kernelHeight - strideHeight;
 
   padWLeft = totalVerticalPadding / 2;
   padWRight = totalVerticalPadding - totalVerticalPadding / 2;
