@@ -53,43 +53,122 @@ TransposedConvolution<
 >::TransposedConvolution(
     const size_t inSize,
     const size_t outSize,
-    const size_t kW,
-    const size_t kH,
-    const size_t dW,
-    const size_t dH,
+    const size_t kernelWidth,
+    const size_t kernelHeight,
+    const size_t strideWidth,
+    const size_t strideHeight,
     const size_t padW,
     const size_t padH,
     const size_t inputWidth,
     const size_t inputHeight,
     const size_t outputWidth,
-    const size_t outputHeight) :
+    const size_t outputHeight,
+    const std::string& paddingType) :
+    TransposedConvolution(
+      inSize,
+      outSize,
+      kernelWidth,
+      kernelHeight,
+      strideWidth,
+      strideHeight,
+      std::tuple<size_t, size_t>(padW, padW),
+      std::tuple<size_t, size_t>(padH, padH),
+      inputWidth,
+      inputHeight,
+      outputWidth,
+      outputHeight,
+      paddingType)
+{
+  // Nothing to do here.
+}
+
+template<
+    typename ForwardConvolutionRule,
+    typename BackwardConvolutionRule,
+    typename GradientConvolutionRule,
+    typename InputDataType,
+    typename OutputDataType
+>
+TransposedConvolution<
+    ForwardConvolutionRule,
+    BackwardConvolutionRule,
+    GradientConvolutionRule,
+    InputDataType,
+    OutputDataType
+>::TransposedConvolution(
+    const size_t inSize,
+    const size_t outSize,
+    const size_t kernelWidth,
+    const size_t kernelHeight,
+    const size_t strideWidth,
+    const size_t strideHeight,
+    const std::tuple<size_t, size_t>& padW,
+    const std::tuple<size_t, size_t>& padH,
+    const size_t inputWidth,
+    const size_t inputHeight,
+    const size_t outputWidth,
+    const size_t outputHeight,
+    const std::string& paddingType) :
     inSize(inSize),
     outSize(outSize),
-    kW(kW),
-    kH(kH),
-    dW(dW),
-    dH(dH),
-    padW(kW - padW - 1),
-    padH(kH - padH - 1),
+    kernelWidth(kernelWidth),
+    kernelHeight(kernelHeight),
+    strideWidth(strideWidth),
+    strideHeight(strideHeight),
+    padWLeft(std::get<0>(padW)),
+    padWRight(std::get<1>(padW)),
+    padHBottom(std::get<1>(padH)),
+    padHTop(std::get<0>(padH)),
     inputWidth(inputWidth),
     inputHeight(inputHeight),
     outputWidth(outputWidth),
     outputHeight(outputHeight)
 {
-  weights.set_size((outSize * inSize * kW * kH) + outSize, 1);
-  // TODO: Use the Padding layer.
-  // padding = new Padding<>(this->padW, this->padW, this->padH, this->padH);
+  weights.set_size((outSize * inSize * kernelWidth * kernelHeight) + outSize,
+      1);
+  // Transform paddingType to lowercase.
+  std::string paddingTypeLow = paddingType;
+  std::transform(paddingType.begin(), paddingType.end(), paddingTypeLow.begin(),
+      [](unsigned char c){ return std::tolower(c); });
 
-  aW = (outputWidth + kW - 2 * this->padW - 2) % dW;
-  aH = (outputHeight + kH - 2 * this->padH - 2) % dH;
+  if (paddingTypeLow == "valid")
+  {
+    // Set Padding to 0.
+    padWLeft = 0;
+    padWRight = 0;
+    padHTop = 0;
+    padHBottom = 0;
+  }
+  else if (paddingTypeLow == "same")
+  {
+    InitializeSamePadding();
+  }
+
+  const size_t totalPadWidth = padWLeft + padWRight;
+  const size_t totalPadHeight = padHTop + padHBottom;
+
+  aW = (outputWidth + totalPadWidth - kernelWidth) % strideWidth;
+  aH = (outputHeight + totalPadHeight - kernelHeight) % strideHeight;
+
+  const size_t padWidthLeftForward = kernelWidth - padWLeft - 1;
+  const size_t padHeightTopForward = kernelHeight - padHTop - 1;
+  const size_t padWidthRightForward = kernelWidth - padWRight - 1;
+  const size_t padHeightBottomtForward = kernelHeight - padHBottom - 1;
+
+  paddingForward = ann::Padding<>(padWidthLeftForward,
+      padWidthRightForward + aW, padHeightTopForward,
+      padHeightBottomtForward + aH);
+  paddingBackward = ann::Padding<>(padWLeft, padWRight, padHTop, padHBottom);
 
   // Check if the output height and width are possible given the other
   // parameters of the layer.
-  if (outputWidth != dW * (inputWidth - 1) + aW + 2 * this->padW + 2 - kW ||
-      outputHeight != dH * (inputHeight - 1) + aH + 2 * this->padH + 2 - kH)
+  if (outputWidth != strideWidth * (inputWidth - 1) +
+      aW + kernelWidth - totalPadWidth ||
+      outputHeight != strideHeight * (inputHeight - 1) +
+      aH + kernelHeight - totalPadHeight)
   {
     Log::Fatal << "The output width / output height is not possible given "
-        << "the other parameters of the layer." << std::endl;
+               << "the other parameters of the layer." << std::endl;
   }
 }
 
@@ -108,7 +187,7 @@ void TransposedConvolution<
     OutputDataType
 >::Reset()
 {
-    weight = arma::cube(weights.memptr(), kW, kH,
+    weight = arma::cube(weights.memptr(), kernelWidth, kernelHeight,
         outSize * inSize, false, false);
     bias = arma::mat(weights.memptr() + weight.n_elem,
         outSize, 1, false, false);
@@ -128,18 +207,30 @@ void TransposedConvolution<
     GradientConvolutionRule,
     InputDataType,
     OutputDataType
->::Forward(const arma::Mat<eT>&& input, arma::Mat<eT>&& output)
+>::Forward(const arma::Mat<eT>& input, arma::Mat<eT>& output)
 {
   batchSize = input.n_cols;
-  inputTemp = arma::cube(const_cast<arma::Mat<eT>&&>(input).memptr(),
+  arma::cube inputTemp(const_cast<arma::Mat<eT>&>(input).memptr(),
       inputWidth, inputHeight, inSize * batchSize, false, false);
 
-  if (dW > 1 || dH > 1)
+  if (strideWidth > 1 || strideHeight > 1)
   {
-    InsertZeros(inputTemp, dW, dH, inputExpandedTemp);
+    InsertZeros(inputTemp, strideWidth, strideHeight, inputExpandedTemp);
 
-    if (padW != 0 || padH != 0 || aW != 0 || aH != 0)
-      Pad(inputExpandedTemp, padW, padH, aW, aH, inputPaddedTemp);
+    if (paddingForward.PadWLeft() != 0 || paddingForward.PadWRight() != 0 ||
+        paddingForward.PadHTop() != 0 || paddingForward.PadHBottom() != 0)
+    {
+      inputPaddedTemp.set_size(inputExpandedTemp.n_rows +
+          paddingForward.PadWLeft() + paddingForward.PadWRight(),
+          inputExpandedTemp.n_cols + paddingForward.PadHTop() +
+          paddingForward.PadHBottom(), inputExpandedTemp.n_slices);
+
+      for (size_t i = 0; i < inputExpandedTemp.n_slices; ++i)
+      {
+        paddingForward.Forward(inputExpandedTemp.slice(i),
+            inputPaddedTemp.slice(i));
+      }
+    }
     else
     {
       inputPaddedTemp = arma::Cube<eT>(inputExpandedTemp.memptr(),
@@ -147,9 +238,20 @@ void TransposedConvolution<
           inputExpandedTemp.n_slices, false, false);;
     }
   }
-  else if (padW != 0 || padH != 0 || aW != 0 || aH != 0)
+  else if (paddingForward.PadWLeft() != 0 ||
+           paddingForward.PadWRight() != 0 ||
+           paddingForward.PadHTop() != 0 ||
+           paddingForward.PadHBottom() != 0)
   {
-    Pad(inputTemp, padW, padH, aW, aH, inputPaddedTemp);
+    inputPaddedTemp.set_size(inputTemp.n_rows + paddingForward.PadWLeft() +
+        paddingForward.PadWRight(), inputTemp.n_cols +
+        paddingForward.PadHTop() + paddingForward.PadHBottom(),
+        inputTemp.n_slices);
+
+    for (size_t i = 0; i < inputTemp.n_slices; ++i)
+    {
+      paddingForward.Forward(inputTemp.slice(i), inputPaddedTemp.slice(i));
+    }
   }
 
   output.set_size(outputWidth * outputHeight * outSize, batchSize);
@@ -171,7 +273,12 @@ void TransposedConvolution<
       arma::Mat<eT> convOutput, rotatedFilter;
       Rotate180(weight.slice(outMapIdx), rotatedFilter);
 
-      if (dW > 1 || dH > 1 || padW != 0 || padH != 0 || aW != 0 || aH != 0)
+      if (strideWidth > 1 ||
+          strideHeight > 1 ||
+          paddingForward.PadWLeft() != 0 ||
+          paddingForward.PadWRight() != 0 ||
+          paddingForward.PadHTop() != 0 ||
+          paddingForward.PadHBottom() != 0)
       {
         ForwardConvolutionRule::Convolution(inputPaddedTemp.slice(inMap +
             batchCount * inSize), rotatedFilter, convOutput, 1, 1);
@@ -204,18 +311,28 @@ void TransposedConvolution<
     InputDataType,
     OutputDataType
 >::Backward(
-    const arma::Mat<eT>&& /* input */, arma::Mat<eT>&& gy, arma::Mat<eT>&& g)
+    const arma::Mat<eT>& /* input */, const arma::Mat<eT>& gy, arma::Mat<eT>& g)
 {
-  arma::Cube<eT> mappedError(gy.memptr(), outputWidth, outputHeight,
-      outSize * batchSize, false, false);
-
+  arma::Cube<eT> mappedError(((arma::Mat<eT>&) gy).memptr(), outputWidth,
+      outputHeight, outSize * batchSize, false, false);
   arma::Cube<eT> mappedErrorPadded;
-  if ((int)(kW - padW - 1) > 0 || (int)(kH - padH - 1) > 0)
-    Pad(mappedError, kW - padW - 1, kH - padH - 1, 0, 0, mappedErrorPadded);
+  if (paddingBackward.PadWLeft() != 0 || paddingBackward.PadWRight() != 0 ||
+      paddingBackward.PadHTop() != 0 || paddingBackward.PadHBottom() != 0)
+  {
+    mappedErrorPadded.set_size(mappedError.n_rows +
+        paddingBackward.PadWLeft() + paddingBackward.PadWRight(),
+        mappedError.n_cols + paddingBackward.PadHTop() +
+        paddingBackward.PadHBottom(), mappedError.n_slices);
 
-  g.set_size(inputTemp.n_rows * inputTemp.n_cols * inSize, batchSize);
-  gTemp = arma::Cube<eT>(g.memptr(), inputTemp.n_rows,
-      inputTemp.n_cols, inputTemp.n_slices, false, false);
+    for (size_t i = 0; i < mappedError.n_slices; ++i)
+    {
+      paddingBackward.Forward(mappedError.slice(i),
+          mappedErrorPadded.slice(i));
+    }
+  }
+  g.set_size(inputWidth * inputHeight * inSize, batchSize);
+  gTemp = arma::Cube<eT>(g.memptr(), inputWidth, inputHeight, inSize *
+      batchSize, false, false);
 
   gTemp.zeros();
 
@@ -232,15 +349,16 @@ void TransposedConvolution<
     {
       arma::Mat<eT> output;
 
-      if ((int)(kW - padW - 1) > 0 || (int)(kH - padH - 1) > 0)
+      if (paddingBackward.PadWLeft() != 0 || paddingBackward.PadWRight() != 0 ||
+          paddingBackward.PadHTop() != 0 || paddingBackward.PadHBottom() != 0)
       {
         BackwardConvolutionRule::Convolution(mappedErrorPadded.slice(outMap),
-            weight.slice(outMapIdx), output, dW, dH);
+            weight.slice(outMapIdx), output, strideWidth, strideHeight);
       }
       else
       {
         BackwardConvolutionRule::Convolution(mappedError.slice(outMap),
-            weight.slice(outMapIdx), output, dW, dH);
+            weight.slice(outMapIdx), output, strideWidth, strideHeight);
       }
 
       gTemp.slice(inMap + batchCount * inSize) += output;
@@ -263,12 +381,14 @@ void TransposedConvolution<
     InputDataType,
     OutputDataType
 >::Gradient(
-    const arma::Mat<eT>&& /* input */,
-    arma::Mat<eT>&& error,
-    arma::Mat<eT>&& gradient)
+    const arma::Mat<eT>& input,
+    const arma::Mat<eT>& error,
+    arma::Mat<eT>& gradient)
 {
-  arma::Cube<eT> mappedError(error.memptr(), outputWidth,
+  arma::Cube<eT> mappedError(((arma::Mat<eT>&) error).memptr(), outputWidth,
       outputHeight, outSize * batchSize, false, false);
+  arma::cube inputTemp(const_cast<arma::Mat<eT>&>(input).memptr(),
+      inputWidth, inputHeight, inSize * batchSize, false, false);
 
   gradient.set_size(weights.n_elem, 1);
   gradientTemp = arma::Cube<eT>(gradient.memptr(), weight.n_rows,
@@ -290,7 +410,12 @@ void TransposedConvolution<
 
     for (size_t inMap = 0; inMap < inSize; inMap++, outMapIdx++)
     {
-      if (dW > 1 || dH > 1 || padW != 0 || padH != 0 || aW != 0 || aH != 0)
+      if (strideWidth > 1 ||
+          strideHeight > 1 ||
+          paddingForward.PadWLeft() != 0 ||
+          paddingForward.PadWRight() != 0 ||
+          paddingForward.PadHTop() != 0 ||
+          paddingForward.PadHBottom() != 0)
       {
         inputSlice = inputPaddedTemp.slice(inMap + batchCount * inSize);
       }
@@ -325,28 +450,84 @@ void TransposedConvolution<
     InputDataType,
     OutputDataType
 >::serialize(
-    Archive& ar, const unsigned int /* version */)
+    Archive& ar, const unsigned int version)
 {
   ar & BOOST_SERIALIZATION_NVP(inSize);
   ar & BOOST_SERIALIZATION_NVP(outSize);
   ar & BOOST_SERIALIZATION_NVP(batchSize);
-  ar & BOOST_SERIALIZATION_NVP(kW);
-  ar & BOOST_SERIALIZATION_NVP(kH);
-  ar & BOOST_SERIALIZATION_NVP(dW);
-  ar & BOOST_SERIALIZATION_NVP(dH);
-  ar & BOOST_SERIALIZATION_NVP(padW);
-  ar & BOOST_SERIALIZATION_NVP(padH);
+  ar & BOOST_SERIALIZATION_NVP(kernelWidth);
+  ar & BOOST_SERIALIZATION_NVP(kernelHeight);
+  ar & BOOST_SERIALIZATION_NVP(strideWidth);
+  ar & BOOST_SERIALIZATION_NVP(strideHeight);
+  if (version == 0)
+  {
+    // These are now stored in paddingForward and paddingBackward.
+    size_t padWidth, padHeight;
+    ar & BOOST_SERIALIZATION_NVP(padWidth);
+    ar & BOOST_SERIALIZATION_NVP(padHeight);
+  }
+  ar &BOOST_SERIALIZATION_NVP(padWLeft);
+  ar &BOOST_SERIALIZATION_NVP(padWRight);
+  ar &BOOST_SERIALIZATION_NVP(padHBottom);
+  ar &BOOST_SERIALIZATION_NVP(padHTop);
   ar & BOOST_SERIALIZATION_NVP(inputWidth);
   ar & BOOST_SERIALIZATION_NVP(inputHeight);
   ar & BOOST_SERIALIZATION_NVP(outputWidth);
   ar & BOOST_SERIALIZATION_NVP(outputHeight);
 
+  if (version > 0)
+  {
+    ar & BOOST_SERIALIZATION_NVP(paddingForward);
+    ar & BOOST_SERIALIZATION_NVP(paddingBackward);
+  }
+
   if (Archive::is_loading::value)
   {
-    weights.set_size((outSize * inSize * kW * kH) + outSize, 1);
+    weights.set_size((outSize * inSize * kernelWidth * kernelHeight) + outSize,
+        1);
+    size_t totalPadWidth = padWLeft + padWRight;
+    size_t totalPadHeight = padHTop + padHBottom;
+    aW = (outputWidth + kernelWidth - totalPadWidth - 2) % strideWidth;
+    aH = (outputHeight + kernelHeight - totalPadHeight - 2) % strideHeight;
+  }
+}
+template<
+    typename ForwardConvolutionRule,
+    typename BackwardConvolutionRule,
+    typename GradientConvolutionRule,
+    typename InputDataType,
+    typename OutputDataType
+>
+void TransposedConvolution<
+    ForwardConvolutionRule,
+    BackwardConvolutionRule,
+    GradientConvolutionRule,
+    InputDataType,
+    OutputDataType
+>::InitializeSamePadding(){
+  /**
+   * Using O=s*(I-1) + K -2P + A
+   * where
+   * s=stride 
+   * I=Input Shape
+   * K=Kernel Size
+   * P=Padding
+   */
+  const size_t totalHorizontalPadding  = (strideWidth - 1) * inputWidth +
+      kernelWidth - strideWidth;
+  const size_t totalVerticalPadding = (strideHeight - 1) * inputHeight +
+      kernelHeight - strideHeight;
 
-    aW = (outputWidth + kW - 2 * padW - 2) % dW;
-    aH = (outputHeight + kH - 2 * padH - 2) % dH;
+  padWLeft = totalVerticalPadding / 2;
+  padWRight = totalVerticalPadding - totalVerticalPadding / 2;
+  padHTop = totalHorizontalPadding / 2;
+  padHBottom = totalHorizontalPadding - totalHorizontalPadding / 2;
+
+  // If Padding is negative throw a fatal error.
+  if (totalHorizontalPadding < 0 || totalVerticalPadding < 0)
+  {
+    Log::Fatal << "The output width / output height is not possible given "
+               << "same padding for the layer." << std::endl;
   }
 }
 
