@@ -61,6 +61,40 @@ RNN<OutputLayerType, InitializationRuleType, CustomLayers...>::~RNN()
   }
 }
 
+template<typename OutputLayerType, typename InitializationRuleType,
+         typename... CustomLayers>
+template<typename OptimizerType>
+typename std::enable_if<
+      HasMaxIterations<OptimizerType, size_t&(OptimizerType::*)()>
+      ::value, void>::type
+RNN<OutputLayerType, InitializationRuleType, CustomLayers...>::
+WarnMessageMaxIterations(OptimizerType& optimizer, size_t samples) const
+{
+  if (optimizer.MaxIterations() < samples &&
+      optimizer.MaxIterations() != 0)
+  {
+    Log::Warn << "The optimizer's maximum number of iterations "
+              << "is less than the size of the dataset; the "
+              << "optimizer will not pass over the entire "
+              << "dataset. To fix this, modify the maximum "
+              << "number of iterations to be at least equal "
+              << "to the number of points of your dataset "
+              << "(" << samples << ")." << std::endl;
+  }
+}
+
+template<typename OutputLayerType, typename InitializationRuleType,
+         typename... CustomLayers>
+template<typename OptimizerType>
+typename std::enable_if<
+      !HasMaxIterations<OptimizerType, size_t&(OptimizerType::*)()>
+      ::value, void>::type
+RNN<OutputLayerType, InitializationRuleType, CustomLayers...>::
+WarnMessageMaxIterations(OptimizerType& optimizer, size_t samples) const
+{
+  return;
+}
+
 template<typename OutputLayerType, typename InitializationRuleType,
          typename... CustomLayers>
 template<typename OptimizerType, typename... CallbackTypes>
@@ -83,6 +117,8 @@ double RNN<OutputLayerType, InitializationRuleType, CustomLayers...>::Train(
     ResetParameters();
   }
 
+  WarnMessageMaxIterations<OptimizerType>(optimizer, this->predictors.n_cols);
+
   // Train the model.
   Timer::Start("rnn_optimization");
   const double out = optimizer.Optimize(*this, parameter, callbacks...);
@@ -127,6 +163,8 @@ double RNN<OutputLayerType, InitializationRuleType, CustomLayers...>::Train(
 
   OptimizerType optimizer;
 
+  WarnMessageMaxIterations<OptimizerType>(optimizer, this->predictors.n_cols);
+
   // Train the model.
   Timer::Start("rnn_optimization");
   const double out = optimizer.Optimize(*this, parameter, callbacks...);
@@ -158,8 +196,8 @@ void RNN<OutputLayerType, InitializationRuleType, CustomLayers...>::Predict(
   const size_t effectiveBatchSize = std::min(batchSize,
       size_t(predictors.n_cols));
 
-  Forward(std::move(arma::mat(predictors.slice(0).colptr(0),
-      predictors.n_rows, effectiveBatchSize, false, true)));
+  Forward(arma::mat(predictors.slice(0).colptr(0), predictors.n_rows,
+      effectiveBatchSize, false, true));
   arma::mat resultsTemp = boost::apply_visitor(outputParameterVisitor,
       network.back());
 
@@ -175,8 +213,8 @@ void RNN<OutputLayerType, InitializationRuleType, CustomLayers...>::Predict(
         size_t(predictors.n_cols - begin));
     for (size_t seqNum = !begin; seqNum < rho; ++seqNum)
     {
-      Forward(std::move(arma::mat(predictors.slice(seqNum).colptr(begin),
-          predictors.n_rows, effectiveBatchSize, false, true)));
+      Forward(arma::mat(predictors.slice(seqNum).colptr(begin),
+          predictors.n_rows, effectiveBatchSize, false, true));
 
       results.slice(seqNum).submat(0, begin, results.n_rows - 1, begin +
           effectiveBatchSize - 1) = boost::apply_visitor(outputParameterVisitor,
@@ -224,16 +262,16 @@ double RNN<OutputLayerType, InitializationRuleType, CustomLayers...>::Evaluate(
     // Wrap a matrix around our data to avoid a copy.
     arma::mat stepData(predictors.slice(seqNum).colptr(begin),
         predictors.n_rows, batchSize, false, true);
-    Forward(std::move(stepData));
+    Forward(stepData);
     if (!single)
     {
       responseSeq = seqNum;
     }
 
-    performance += outputLayer.Forward(std::move(boost::apply_visitor(
-        outputParameterVisitor, network.back())),
-        std::move(arma::mat(responses.slice(responseSeq).colptr(begin),
-            responses.n_rows, batchSize, false, true)));
+    performance += outputLayer.Forward(boost::apply_visitor(
+        outputParameterVisitor, network.back()),
+        arma::mat(responses.slice(responseSeq).colptr(begin),
+            responses.n_rows, batchSize, false, true));
   }
 
   if (outputSize == 0)
@@ -299,13 +337,14 @@ EvaluateWithGradient(const arma::mat& /* parameters */,
 
   double performance = 0;
   size_t responseSeq = 0;
+  const size_t effectiveRho = std::min(rho, size_t(responses.size()));
 
-  for (size_t seqNum = 0; seqNum < rho; ++seqNum)
+  for (size_t seqNum = 0; seqNum < effectiveRho; ++seqNum)
   {
     // Wrap a matrix around our data to avoid a copy.
     arma::mat stepData(predictors.slice(seqNum).colptr(begin),
         predictors.n_rows, batchSize, false, true);
-    Forward(std::move(stepData));
+    Forward(stepData);
     if (!single)
     {
       responseSeq = seqNum;
@@ -313,14 +352,14 @@ EvaluateWithGradient(const arma::mat& /* parameters */,
 
     for (size_t l = 0; l < network.size(); ++l)
     {
-      boost::apply_visitor(SaveOutputParameterVisitor(
-          std::move(moduleOutputParameter)), network[l]);
+      boost::apply_visitor(SaveOutputParameterVisitor(moduleOutputParameter),
+          network[l]);
     }
 
-    performance += outputLayer.Forward(std::move(boost::apply_visitor(
-        outputParameterVisitor, network.back())),
-        std::move(arma::mat(responses.slice(responseSeq).colptr(begin),
-            responses.n_rows, batchSize, false, true)));
+    performance += outputLayer.Forward(boost::apply_visitor(
+        outputParameterVisitor, network.back()),
+        arma::mat(responses.slice(responseSeq).colptr(begin),
+            responses.n_rows, batchSize, false, true));
   }
 
   if (outputSize == 0)
@@ -338,14 +377,13 @@ EvaluateWithGradient(const arma::mat& /* parameters */,
 
   ResetGradients(currentGradient);
 
-  for (size_t seqNum = 0; seqNum < rho; ++seqNum)
+  for (size_t seqNum = 0; seqNum < effectiveRho; ++seqNum)
   {
     currentGradient.zeros();
-
     for (size_t l = 0; l < network.size(); ++l)
     {
-      boost::apply_visitor(LoadOutputParameterVisitor(
-          std::move(moduleOutputParameter)), network[network.size() - 1 - l]);
+      boost::apply_visitor(LoadOutputParameterVisitor(moduleOutputParameter),
+          network[network.size() - 1 - l]);
     }
 
     if (single && seqNum > 0)
@@ -354,23 +392,23 @@ EvaluateWithGradient(const arma::mat& /* parameters */,
     }
     else if (single && seqNum == 0)
     {
-      outputLayer.Backward(std::move(boost::apply_visitor(
-          outputParameterVisitor, network.back())),
-          std::move(arma::mat(responses.slice(0).colptr(begin),
-          responses.n_rows, batchSize, false, true)), std::move(error));
+      outputLayer.Backward(boost::apply_visitor(
+          outputParameterVisitor, network.back()),
+          arma::mat(responses.slice(0).colptr(begin),
+          responses.n_rows, batchSize, false, true), error);
     }
     else
     {
-      outputLayer.Backward(std::move(boost::apply_visitor(
-          outputParameterVisitor, network.back())),
-          std::move(arma::mat(responses.slice(rho - seqNum - 1).colptr(begin),
-          responses.n_rows, batchSize, false, true)), std::move(error));
+      outputLayer.Backward(boost::apply_visitor(
+          outputParameterVisitor, network.back()),
+          arma::mat(responses.slice(effectiveRho - seqNum - 1).colptr(begin),
+          responses.n_rows, batchSize, false, true), error);
     }
 
     Backward();
-    Gradient(std::move(
-        arma::mat(predictors.slice(rho - seqNum - 1).colptr(begin),
-        predictors.n_rows, batchSize, false, true)));
+    Gradient(
+        arma::mat(predictors.slice(effectiveRho - seqNum - 1).colptr(begin),
+        predictors.n_rows, batchSize, false, true));
     gradient += currentGradient;
   }
 
@@ -443,25 +481,25 @@ void RNN<OutputLayerType, InitializationRuleType,
   size_t offset = 0;
   for (LayerTypes<CustomLayers...>& layer : network)
   {
-    offset += boost::apply_visitor(GradientSetVisitor(std::move(gradient),
-        offset), layer);
+    offset += boost::apply_visitor(GradientSetVisitor(gradient, offset), layer);
   }
 }
 
 template<typename OutputLayerType, typename InitializationRuleType,
          typename... CustomLayers>
+template<typename InputType>
 void RNN<OutputLayerType, InitializationRuleType,
-         CustomLayers...>::Forward(arma::mat&& input)
+         CustomLayers...>::Forward(const InputType& input)
 {
-  boost::apply_visitor(ForwardVisitor(std::move(input), std::move(
-      boost::apply_visitor(outputParameterVisitor, network.front()))),
+  boost::apply_visitor(ForwardVisitor(input,
+      boost::apply_visitor(outputParameterVisitor, network.front())),
       network.front());
 
   for (size_t i = 1; i < network.size(); ++i)
   {
     boost::apply_visitor(ForwardVisitor(
-        std::move(boost::apply_visitor(outputParameterVisitor, network[i - 1])),
-        std::move(boost::apply_visitor(outputParameterVisitor, network[i]))),
+        boost::apply_visitor(outputParameterVisitor, network[i - 1]),
+        boost::apply_visitor(outputParameterVisitor, network[i])),
         network[i]);
   }
 }
@@ -471,17 +509,17 @@ template<typename OutputLayerType, typename InitializationRuleType,
 void RNN<OutputLayerType, InitializationRuleType, CustomLayers...>::Backward()
 {
   boost::apply_visitor(BackwardVisitor(
-        std::move(boost::apply_visitor(outputParameterVisitor, network.back())),
-        std::move(error), std::move(boost::apply_visitor(deltaVisitor,
-        network.back()))), network.back());
+        boost::apply_visitor(outputParameterVisitor, network.back()),
+        error, boost::apply_visitor(deltaVisitor,
+        network.back())), network.back());
 
   for (size_t i = 2; i < network.size(); ++i)
   {
     boost::apply_visitor(BackwardVisitor(
-        std::move(boost::apply_visitor(outputParameterVisitor,
-        network[network.size() - i])), std::move(boost::apply_visitor(
-        deltaVisitor, network[network.size() - i + 1])), std::move(
-        boost::apply_visitor(deltaVisitor, network[network.size() - i]))),
+        boost::apply_visitor(outputParameterVisitor,
+        network[network.size() - i]), boost::apply_visitor(
+        deltaVisitor, network[network.size() - i + 1]),
+        boost::apply_visitor(deltaVisitor, network[network.size() - i])),
         network[network.size() - i]);
   }
 }
@@ -490,16 +528,16 @@ template<typename OutputLayerType, typename InitializationRuleType,
          typename... CustomLayers>
 template<typename InputType>
 void RNN<OutputLayerType, InitializationRuleType,
-         CustomLayers...>::Gradient(InputType&& input)
+         CustomLayers...>::Gradient(const InputType& input)
 {
-  boost::apply_visitor(GradientVisitor(std::move(input), std::move(
-      boost::apply_visitor(deltaVisitor, network[1]))), network.front());
+  boost::apply_visitor(GradientVisitor(input,
+      boost::apply_visitor(deltaVisitor, network[1])), network.front());
 
   for (size_t i = 1; i < network.size() - 1; ++i)
   {
     boost::apply_visitor(GradientVisitor(
-        std::move(boost::apply_visitor(outputParameterVisitor, network[i - 1])),
-        std::move(boost::apply_visitor(deltaVisitor, network[i + 1]))),
+        boost::apply_visitor(outputParameterVisitor, network[i - 1]),
+        boost::apply_visitor(deltaVisitor, network[i + 1])),
         network[i]);
   }
 }
@@ -543,8 +581,8 @@ void RNN<OutputLayerType, InitializationRuleType, CustomLayers...>::serialize(
     size_t offset = 0;
     for (LayerTypes<CustomLayers...>& layer : network)
     {
-      offset += boost::apply_visitor(WeightSetVisitor(std::move(parameter),
-          offset), layer);
+      offset += boost::apply_visitor(WeightSetVisitor(parameter, offset),
+          layer);
 
       boost::apply_visitor(resetVisitor, layer);
     }
