@@ -20,12 +20,28 @@ namespace ann {
 
 //! DeleteVisitor visitor class.
 template<typename LayerType>
-inline void DeleteVisitor::operator()(LayerType* layer) const
+inline typename std::enable_if<
+    !HasModelCheck<LayerType>::value, void>::type
+DeleteVisitor::operator()(LayerType* layer) const
 {
   if (layer)
     delete layer;
 }
 
+template<typename LayerType>
+inline typename std::enable_if<
+    HasModelCheck<LayerType>::value, void>::type
+DeleteVisitor::operator()(LayerType* layer) const
+{
+  if (layer)
+  {
+    for (size_t i = 0; i < layer->Model().size(); ++i)
+      boost::apply_visitor(DeleteVisitor(), layer->Model()[i]);
+
+    delete layer;
+  }
+}
+
 inline void DeleteVisitor::operator()(MoreTypes layer) const
 {
   layer.apply_visitor(*this);
