@@ -29,7 +29,7 @@ class ForwardVisitor : public boost::static_visitor<void>
 {
  public:
   //! Execute the Forward() function given the input and output parameter.
-  ForwardVisitor(arma::mat&& input, arma::mat&& output);
+  ForwardVisitor(const arma::mat& input, arma::mat& output);
 
   //! Execute the Forward() function.
   template<typename LayerType>
@@ -39,10 +39,10 @@ class ForwardVisitor : public boost::static_visitor<void>
 
  private:
   //! The input parameter set.
-  arma::mat&& input;
+  const arma::mat& input;
 
   //! The output parameter set.
-  arma::mat&& output;
+  arma::mat& output;
 };
 
 } // namespace ann
