@@ -19,9 +19,9 @@ namespace mlpack {
 namespace ann {
 
 //! GradientSetVisitor visitor class.
-inline GradientSetVisitor::GradientSetVisitor(arma::mat&& gradient,
+inline GradientSetVisitor::GradientSetVisitor(arma::mat& gradient,
                                               size_t offset) :
-    gradient(std::move(gradient)),
+    gradient(gradient),
     offset(offset)
 {
   /* Nothing to do here. */
@@ -60,7 +60,7 @@ GradientSetVisitor::LayerGradients(T* layer, arma::mat& /* input */) const
   for (size_t i = 0; i < layer->Model().size(); ++i)
   {
     modelOffset += boost::apply_visitor(GradientSetVisitor(
-        std::move(gradient), modelOffset + offset), layer->Model()[i]);
+        gradient, modelOffset + offset), layer->Model()[i]);
   }
 
   return modelOffset;
@@ -79,7 +79,7 @@ GradientSetVisitor::LayerGradients(T* layer, arma::mat& /* input */) const
   for (size_t i = 0; i < layer->Model().size(); ++i)
   {
     modelOffset += boost::apply_visitor(GradientSetVisitor(
-        std::move(gradient), modelOffset + offset), layer->Model()[i]);
+        gradient, modelOffset + offset), layer->Model()[i]);
   }
 
   return modelOffset;
