@@ -30,15 +30,20 @@ HMM<Distribution>::HMM(const size_t states,
                        const Distribution emissions,
                        const double tolerance) :
     emission(states, /* default distribution */ emissions),
-    transition(arma::randu<arma::mat>(states, states)),
-    initial(arma::randu<arma::vec>(states) / (double) states),
+    transitionProxy(arma::randu<arma::mat>(states, states)),
+    initialProxy(arma::randu<arma::vec>(states) / (double) states),
     dimensionality(emissions.Dimensionality()),
-    tolerance(tolerance)
+    tolerance(tolerance),
+    recalculateInitial(false),
+    recalculateTransition(false)
 {
   // Normalize the transition probabilities and initial state probabilities.
-  initial /= arma::accu(initial);
-  for (size_t i = 0; i < transition.n_cols; ++i)
-    transition.col(i) /= arma::accu(transition.col(i));
+  initialProxy /= arma::accu(initialProxy);
+  for (size_t i = 0; i < transitionProxy.n_cols; ++i)
+    transitionProxy.col(i) /= arma::accu(transitionProxy.col(i));
+
+  logTransition = log(transitionProxy);
+  logInitial = log(initialProxy);
 }
 
 /**
@@ -51,9 +56,13 @@ HMM<Distribution>::HMM(const arma::vec& initial,
                        const std::vector<Distribution>& emission,
                        const double tolerance) :
     emission(emission),
-    transition(transition),
-    initial(initial),
-    tolerance(tolerance)
+    transitionProxy(transition),
+    logTransition(log(transition)),
+    initialProxy(initial),
+    logInitial(log(initial)),
+    tolerance(tolerance),
+    recalculateInitial(false),
+    recalculateTransition(false)
 {
   // Set the dimensionality, if we can.
   if (emission.size() > 0)
@@ -106,7 +115,7 @@ double HMM<Distribution>::Train(const std::vector<arma::mat>& dataSeq)
 
   // These are used later for training of each distribution.  We initialize it
   // all now so we don't have to do any allocation later on.
-  std::vector<arma::vec> emissionProb(transition.n_cols,
+  std::vector<arma::vec> emissionProb(logTransition.n_cols,
       arma::vec(totalLength));
   arma::mat emissionList(dimensionality, totalLength);
 
@@ -116,9 +125,9 @@ double HMM<Distribution>::Train(const std::vector<arma::mat>& dataSeq)
   for (size_t iter = 0; iter < iterations; iter++)
   {
     // Clear new transition matrix and emission probabilities.
-    arma::vec newLogInitial(transition.n_rows);
+    arma::vec newLogInitial(logTransition.n_rows);
     newLogInitial.fill(-std::numeric_limits<double>::infinity());
-    arma::mat newLogTransition(transition.n_rows, transition.n_cols);
+    arma::mat newLogTransition(logTransition.n_rows, logTransition.n_cols);
     newLogTransition.fill(-std::numeric_limits<double>::infinity());
 
     // Reset log likelihood.
@@ -140,7 +149,7 @@ double HMM<Distribution>::Train(const std::vector<arma::mat>& dataSeq)
           backwardLog, logScales);
 
       // Add to estimate of initial probability for state j.
-      for (size_t j = 0; j < transition.n_cols; ++j)
+      for (size_t j = 0; j < logTransition.n_cols; ++j)
         newLogInitial[j] = math::LogAdd(newLogInitial[j], stateLogProb(j, 0));
 
       // Now re-estimate the parameters.  This is the M-step.
@@ -151,13 +160,13 @@ double HMM<Distribution>::Train(const std::vector<arma::mat>& dataSeq)
       // We store the new estimates in a different matrix.
       for (size_t t = 0; t < dataSeq[seq].n_cols; ++t)
       {
-        for (size_t j = 0; j < transition.n_cols; ++j)
+        for (size_t j = 0; j < logTransition.n_cols; ++j)
         {
           if (t < dataSeq[seq].n_cols - 1)
           {
             // Estimate of T_ij (probability of transition from state j to state
             // i).  We postpone multiplication of the old T_ij until later.
-            for (size_t i = 0; i < transition.n_rows; i++)
+            for (size_t i = 0; i < logTransition.n_rows; i++)
             {
               newLogTransition(i, j) = math::LogAdd(newLogTransition(i, j),
                   forwardLog(j, t) + backwardLog(i, t + 1) +
@@ -184,28 +193,30 @@ double HMM<Distribution>::Train(const std::vector<arma::mat>& dataSeq)
 
     // Normalize the new initial probabilities.
     if (dataSeq.size() > 1)
-      initial = exp(newLogInitial) / dataSeq.size();
+      logInitial = newLogInitial - log(dataSeq.size());
     else
-      initial = exp(newLogInitial);
+      logInitial = newLogInitial;
 
     // Assign the new transition matrix.  We use %= (element-wise
     // multiplication) because every element of the new transition matrix must
     // still be multiplied by the old elements (this is the multiplication we
     // earlier postponed).
-    transition %= exp(newLogTransition);
+    logTransition += newLogTransition;
 
     // Now we normalize the transition matrix.
-    for (size_t i = 0; i < transition.n_cols; i++)
+    for (size_t i = 0; i < logTransition.n_cols; i++)
     {
-      const double sum = accu(transition.col(i));
-      if (sum > 0.0)
-        transition.col(i) /= sum;
+      const double sum = math::AccuLog(logTransition.col(i));
+      if (std::isfinite(sum))
+        logTransition.col(i) -= sum;
       else
-        transition.col(i).fill(1.0 / (double) transition.n_rows);
+        logTransition.col(i).fill(-log((double) logTransition.n_rows));
     }
 
+    initialProxy = exp(logInitial);
+    transitionProxy = exp(logTransition);
     // Now estimate emission probabilities.
-    for (size_t state = 0; state < transition.n_cols; state++)
+    for (size_t state = 0; state < logTransition.n_cols; state++)
       emission[state].Train(emissionList, emissionProb[state]);
 
     Log::Debug << "Iteration " << iter << ": log-likelihood " << loglik
@@ -230,8 +241,9 @@ void HMM<Distribution>::Train(const std::vector<arma::mat>& dataSeq,
         << ")." << std::endl;
   }
 
-  initial.zeros();
-  transition.zeros();
+  arma::mat initial = arma::zeros(logInitial.n_elem);
+  arma::mat transition = arma::zeros(logTransition.n_rows,
+                                     logTransition.n_cols);
 
   // Estimate the transition and emission matrices directly from the
   // observations.  The emission list holds the time indices for observations
@@ -284,6 +296,11 @@ void HMM<Distribution>::Train(const std::vector<arma::mat>& dataSeq,
       transition.col(col) /= sum;
   }
 
+  initialProxy = initial;
+  transitionProxy = transition;
+  logTransition = log(transition);
+  logInitial = log(initial);
+
   // Estimate emission matrix.
   for (size_t state = 0; state < transition.n_cols; state++)
   {
@@ -406,6 +423,8 @@ void HMM<Distribution>::Generate(const size_t length,
   // distribution of emissions for our starting state.
   dataSequence.col(0) = emission[startState].Random();
 
+  ConvertToLogSpace();
+
   // Now choose the states and emissions for the rest of the sequence.
   for (size_t t = 1; t < length; t++)
   {
@@ -415,9 +434,9 @@ void HMM<Distribution>::Generate(const size_t length,
     // Now find where our random value sits in the probability distribution of
     // state changes.
     double probSum = 0;
-    for (size_t st = 0; st < transition.n_rows; st++)
+    for (size_t st = 0; st < logTransition.n_rows; st++)
     {
-      probSum += transition(st, stateSequence[t - 1]);
+      probSum += exp(logTransition(st, stateSequence[t - 1]));
       if (randValue <= probSum)
       {
         stateSequence[t] = st;
@@ -444,20 +463,18 @@ double HMM<Distribution>::Predict(const arma::mat& dataSeq,
   // don't use log-likelihoods to save that little bit of time, but we'll
   // calculate the log-likelihood at the end of it all.
   stateSeq.set_size(dataSeq.n_cols);
-  arma::mat logStateProb(transition.n_rows, dataSeq.n_cols);
-  arma::mat stateSeqBack(transition.n_rows, dataSeq.n_cols);
+  arma::mat logStateProb(logTransition.n_rows, dataSeq.n_cols);
+  arma::mat stateSeqBack(logTransition.n_rows, dataSeq.n_cols);
 
-  // Store the logs of the transposed transition matrix.  This is because we
-  // will be using the rows of the transition matrix.
-  arma::mat logTrans(log(trans(transition)));
+  ConvertToLogSpace();
 
   // The calculation of the first state is slightly different; the probability
   // of the first state being state j is the maximum probability that the state
   // came to be j from another state.
   logStateProb.col(0).zeros();
-  for (size_t state = 0; state < transition.n_rows; state++)
+  for (size_t state = 0; state < logTransition.n_rows; state++)
   {
-    logStateProb(state, 0) = log(initial[state]) +
+    logStateProb(state, 0) = logInitial[state] +
         emission[state].LogProbability(dataSeq.unsafe_col(0));
     stateSeqBack(state, 0) = state;
   }
@@ -469,9 +486,9 @@ double HMM<Distribution>::Predict(const arma::mat& dataSeq,
     // Assemble the state probability for this element.
     // Given that we are in state j, we use state with the highest probability
     // of being the previous state.
-    for (size_t j = 0; j < transition.n_rows; j++)
+    for (size_t j = 0; j < logTransition.n_rows; j++)
     {
-      arma::vec prob = logStateProb.col(t - 1) + logTrans.col(j);
+      arma::vec prob = logStateProb.col(t - 1) + logTransition.row(j).t();
       logStateProb(j, t) = prob.max(index) +
           emission[j].LogProbability(dataSeq.unsafe_col(t));
       stateSeqBack(j, t) = index;
@@ -518,11 +535,11 @@ void HMM<Distribution>::Filter(const arma::mat& dataSeq,
   arma::vec logScales;
   Forward(dataSeq, logScales, forwardLogProb);
 
-  arma::mat forwardProb = exp(forwardLogProb);
-
   // Propagate state ahead.
   if (ahead != 0)
-    forwardProb = pow(transition, ahead) * forwardProb;
+    forwardLogProb += ahead * logTransition;
+
+  arma::mat forwardProb = exp(forwardLogProb);
 
   // Compute expected emissions.
   // Will not work for distributions without a Mean() function.
@@ -563,21 +580,21 @@ void HMM<Distribution>::Forward(const arma::mat& dataSeq,
 {
   // Our goal is to calculate the forward probabilities:
   //  P(X_k | o_{1:k}) for all possible states X_k, for each time point k.
-  forwardLogProb.resize(transition.n_rows, dataSeq.n_cols);
+  forwardLogProb.resize(logTransition.n_rows, dataSeq.n_cols);
   forwardLogProb.fill(-std::numeric_limits<double>::infinity());
   logScales.resize(dataSeq.n_cols);
   logScales.fill(-std::numeric_limits<double>::infinity());
 
-  arma::mat logTrans = trans(log(transition));
+  ConvertToLogSpace();
 
   // The first entry in the forward algorithm uses the initial state
   // probabilities.  Note that MATLAB assumes that the starting state (at
   // t = -1) is state 0; this is not our assumption here.  To force that
   // behavior, you could append a single starting state to every single data
   // sequence and that should produce results in line with MATLAB.
-  for (size_t state = 0; state < transition.n_rows; state++)
+  for (size_t state = 0; state < logTransition.n_rows; state++)
   {
-    forwardLogProb(state, 0) = log(initial(state)) +
+    forwardLogProb(state, 0) = logInitial(state) +
         emission[state].LogProbability(dataSeq.unsafe_col(0));
   }
 
@@ -589,12 +606,12 @@ void HMM<Distribution>::Forward(const arma::mat& dataSeq,
   // Now compute the probabilities for each successive observation.
   for (size_t t = 1; t < dataSeq.n_cols; t++)
   {
-    for (size_t j = 0; j < transition.n_rows; j++)
+    for (size_t j = 0; j < logTransition.n_rows; j++)
     {
       // The forward probability of state j at time t is the sum over all states
       // of the probability of the previous state transitioning to the current
       // state and emitting the given observation.
-      arma::vec tmp = forwardLogProb.col(t - 1) + logTrans.col(j);
+      arma::vec tmp = forwardLogProb.col(t - 1) + logTransition.row(j).t();
       forwardLogProb(j, t) = math::AccuLog(tmp) +
           emission[j].LogProbability(dataSeq.unsafe_col(t));
     }
@@ -613,9 +630,8 @@ void HMM<Distribution>::Backward(const arma::mat& dataSeq,
 {
   // Our goal is to calculate the backward probabilities:
   //  P(X_k | o_{k + 1:T}) for all possible states X_k, for each time point k.
-  backwardLogProb.resize(transition.n_rows, dataSeq.n_cols);
+  backwardLogProb.resize(logTransition.n_rows, dataSeq.n_cols);
   backwardLogProb.fill(-std::numeric_limits<double>::infinity());
-  arma::mat logTrans = log(transition);
 
   // The last element probability is 1.
   backwardLogProb.col(dataSeq.n_cols - 1).fill(0);
@@ -623,16 +639,16 @@ void HMM<Distribution>::Backward(const arma::mat& dataSeq,
   // Now step backwards through all other observations.
   for (size_t t = dataSeq.n_cols - 2; t + 1 > 0; t--)
   {
-    for (size_t j = 0; j < transition.n_rows; j++)
+    for (size_t j = 0; j < logTransition.n_rows; j++)
     {
       // The backward probability of state j at time t is the sum over all state
       // of the probability of the next state having been a transition from the
       // current state multiplied by the probability of each of those states
       // emitting the given observation.
-      for (size_t state = 0; state < transition.n_rows; state++)
+      for (size_t state = 0; state < logTransition.n_rows; state++)
       {
         backwardLogProb(j, t) = math::LogAdd(backwardLogProb(j, t),
-            logTrans(state, j) + backwardLogProb(state, t + 1)
+            logTransition(state, j) + backwardLogProb(state, t + 1)
             + emission[state].LogProbability(dataSeq.unsafe_col(t + 1)));
       }
 
@@ -643,11 +659,32 @@ void HMM<Distribution>::Backward(const arma::mat& dataSeq,
   }
 }
 
+/**
+ * Make sure the variables in log space are in sync with the linear counter parts
+ */
+template<typename Distribution>
+void HMM<Distribution>::ConvertToLogSpace() const
+{
+  if (recalculateInitial)
+  {
+    logInitial = log(initialProxy);
+    recalculateInitial = false;
+  }
+
+  if (recalculateTransition)
+  {
+    logTransition = log(transitionProxy);
+    recalculateTransition = false;
+  }
+}
+
 //! Serialize the HMM.
 template<typename Distribution>
 template<typename Archive>
-void HMM<Distribution>::serialize(Archive& ar, const unsigned int /* version */)
+void HMM<Distribution>::load(Archive& ar, const unsigned int /* version */)
 {
+  arma::mat transition;
+  arma::vec initial;
   ar & BOOST_SERIALIZATION_NVP(dimensionality);
   ar & BOOST_SERIALIZATION_NVP(tolerance);
   ar & BOOST_SERIALIZATION_NVP(transition);
@@ -655,11 +692,29 @@ void HMM<Distribution>::serialize(Archive& ar, const unsigned int /* version */)
 
   // Now serialize each emission.  If we are loading, we must resize the vector
   // of emissions correctly.
-  if (Archive::is_loading::value)
-    emission.resize(transition.n_rows);
-
+  emission.resize(transition.n_rows);
   // Load the emissions; generate the correct name for each one.
-    ar & BOOST_SERIALIZATION_NVP(emission);
+  ar & BOOST_SERIALIZATION_NVP(emission);
+
+  logTransition = log(transition);
+  logInitial = log(initial);
+  initialProxy = std::move(initial);
+  transitionProxy = std::move(transition);
+}
+
+//! Serialize the HMM.
+template<typename Distribution>
+template<typename Archive>
+void HMM<Distribution>::save(Archive& ar,
+                             const unsigned int /* version */) const
+{
+  arma::mat transition = exp(logTransition);
+  arma::vec initial = exp(logInitial);
+  ar & BOOST_SERIALIZATION_NVP(dimensionality);
+  ar & BOOST_SERIALIZATION_NVP(tolerance);
+  ar & BOOST_SERIALIZATION_NVP(transition);
+  ar & BOOST_SERIALIZATION_NVP(initial);
+  ar & BOOST_SERIALIZATION_NVP(emission);
 }
 
 } // namespace hmm
