@@ -301,12 +301,13 @@ class NStepQLearningWorker
         target = config.Discount() * target + std::get<2>(transition);
 
         // Compute the training target for current state.
-        network.Forward(std::get<0>(transition).Encode(), actionValue);
+        arma::mat input = std::get<0>(transition).Encode();
+        network.Forward(input, actionValue);
         actionValue[std::get<1>(transition)] = target;
 
         // Compute gradient.
         arma::mat gradients;
-        network.Backward(actionValue, gradients);
+        network.Backward(input, actionValue, gradients);
 
         // Accumulate gradients.
         totalGradients += gradients;
