@@ -21,6 +21,9 @@
 #include <mlpack/methods/ann/activation_functions/softplus_function.hpp>
 #include <mlpack/methods/ann/activation_functions/swish_function.hpp>
 #include <mlpack/methods/ann/activation_functions/hard_sigmoid_function.hpp>
+#include <mlpack/methods/ann/activation_functions/mish_function.hpp>
+#include <mlpack/methods/ann/activation_functions/lisht_function.hpp>
+#include <mlpack/methods/ann/activation_functions/gelu_function.hpp>
 
 #include <boost/test/unit_test.hpp>
 #include "test_tools.hpp"
@@ -130,7 +133,7 @@ void CheckHardTanHActivationCorrect(const arma::colvec input,
 
   // Test the activation function using the entire vector as input.
   arma::colvec activations;
-  htf.Forward(std::move(input), std::move(activations));
+  htf.Forward(input, activations);
   for (size_t i = 0; i < activations.n_elem; i++)
   {
     BOOST_REQUIRE_CLOSE(activations.at(i), target.at(i), 1e-3);
@@ -154,7 +157,7 @@ void CheckHardTanHDerivativeCorrect(const arma::colvec input,
 
   // This error vector will be set to 1 to get the derivatives.
   arma::colvec error = arma::ones<arma::colvec>(input.n_elem);
-  htf.Backward(std::move(input), std::move(error), std::move(derivatives));
+  htf.Backward(input, error, derivatives);
 
   for (size_t i = 0; i < derivatives.n_elem; i++)
   {
@@ -176,7 +179,7 @@ void CheckLeakyReLUActivationCorrect(const arma::colvec input,
 
   // Test the activation function using the entire vector as input.
   arma::colvec activations;
-  lrf.Forward(std::move(input), std::move(activations));
+  lrf.Forward(input, activations);
   for (size_t i = 0; i < activations.n_elem; i++)
   {
     BOOST_REQUIRE_CLOSE(activations.at(i), target.at(i), 1e-3);
@@ -201,7 +204,7 @@ void CheckLeakyReLUDerivativeCorrect(const arma::colvec input,
 
   // This error vector will be set to 1 to get the derivatives.
   arma::colvec error = arma::ones<arma::colvec>(input.n_elem);
-  lrf.Backward(std::move(input), std::move(error), std::move(derivatives));
+  lrf.Backward(input, error, derivatives);
   for (size_t i = 0; i < derivatives.n_elem; i++)
   {
     BOOST_REQUIRE_CLOSE(derivatives.at(i), target.at(i), 1e-3);
@@ -216,14 +219,14 @@ void CheckLeakyReLUDerivativeCorrect(const arma::colvec input,
  * @param target Target data used to evaluate the ELU activation.
  */
 void CheckELUActivationCorrect(const arma::colvec input,
-                                     const arma::colvec target)
+                               const arma::colvec target)
 {
   // Initialize ELU object with alpha = 1.0.
   ELU<> lrf(1.0);
 
   // Test the activation function using the entire vector as input.
   arma::colvec activations;
-  lrf.Forward(std::move(input), std::move(activations));
+  lrf.Forward(input, activations);
   for (size_t i = 0; i < activations.n_elem; i++)
   {
     BOOST_REQUIRE_CLOSE(activations.at(i), target.at(i), 1e-3);
@@ -238,7 +241,7 @@ void CheckELUActivationCorrect(const arma::colvec input,
  * @param target Target data used to evaluate the ELU activation.
  */
 void CheckELUDerivativeCorrect(const arma::colvec input,
-                                     const arma::colvec target)
+                               const arma::colvec target)
 {
   // Initialize ELU object with alpha = 1.0.
   ELU<> lrf(1.0);
@@ -248,9 +251,8 @@ void CheckELUDerivativeCorrect(const arma::colvec input,
 
   // This error vector will be set to 1 to get the derivatives.
   arma::colvec error = arma::ones<arma::colvec>(input.n_elem);
-  lrf.Forward(std::move(input), std::move(activations));
-  lrf.Backward(std::move(activations), std::move(error),
-      std::move(derivatives));
+  lrf.Forward(input, activations);
+  lrf.Backward(activations, error, derivatives);
   for (size_t i = 0; i < derivatives.n_elem; i++)
   {
     BOOST_REQUIRE_CLOSE(derivatives.at(i), target.at(i), 1e-3);
@@ -259,20 +261,20 @@ void CheckELUDerivativeCorrect(const arma::colvec input,
 
 /*
  * Implementation of the PReLU activation function test. The function
- * is implemented as PReLU layer in the file perametric_relu.hpp
+ * is implemented as PReLU layer in the file parametric_relu.hpp.
  *
  * @param input Input data used for evaluating the PReLU activation
  *   function.
  * @param target Target data used to evaluate the PReLU activation.
  */
 void CheckPReLUActivationCorrect(const arma::colvec input,
-                                          const arma::colvec target)
+                                 const arma::colvec target)
 {
   PReLU<> prelu;
 
   // Test the activation function using the entire vector as input.
   arma::colvec activations;
-  prelu.Forward(std::move(input), std::move(activations));
+  prelu.Forward(input, activations);
   for (size_t i = 0; i < activations.n_elem; i++)
   {
     BOOST_REQUIRE_CLOSE(activations.at(i), target.at(i), 1e-3);
@@ -282,14 +284,14 @@ void CheckPReLUActivationCorrect(const arma::colvec input,
 /*
  * Implementation of the PReLU activation function derivative test.
  * The function is implemented as PReLU layer in the file
- * perametric_relu.hpp
+ * parametric_relu.hpp
  *
  * @param input Input data used for evaluating the PReLU activation
  *   function.
  * @param target Target data used to evaluate the PReLU activation.
  */
 void CheckPReLUDerivativeCorrect(const arma::colvec input,
-                                          const arma::colvec target)
+                                 const arma::colvec target)
 {
   PReLU<> prelu;
 
@@ -298,7 +300,7 @@ void CheckPReLUDerivativeCorrect(const arma::colvec input,
 
   // This error vector will be set to 1 to get the derivatives.
   arma::colvec error = arma::ones<arma::colvec>(input.n_elem);
-  prelu.Backward(std::move(input), std::move(error), std::move(derivatives));
+  prelu.Backward(input, error, derivatives);
   for (size_t i = 0; i < derivatives.n_elem; i++)
   {
     BOOST_REQUIRE_CLOSE(derivatives.at(i), target.at(i), 1e-3);
@@ -308,14 +310,14 @@ void CheckPReLUDerivativeCorrect(const arma::colvec input,
 /*
  * Implementation of the PReLU activation function gradient test.
  * The function is implemented as PReLU layer in the file
- * perametric_relu.hpp
+ * parametric_relu.hpp
  *
  * @param input Input data used for evaluating the PReLU activation
  *   function.
  * @param target Target data used to evaluate the PReLU gradient.
  */
 void CheckPReLUGradientCorrect(const arma::colvec input,
-                                        const arma::colvec target)
+                               const arma::colvec target)
 {
   PReLU<> prelu;
 
@@ -324,12 +326,104 @@ void CheckPReLUGradientCorrect(const arma::colvec input,
 
   // This error vector will be set to 1 to get the gradient.
   arma::colvec error = arma::ones<arma::colvec>(input.n_elem);
-  prelu.Gradient(std::move(input), std::move(error), std::move(gradient));
+  prelu.Gradient(input, error, gradient);
   BOOST_REQUIRE_EQUAL(gradient.n_rows, 1);
   BOOST_REQUIRE_EQUAL(gradient.n_cols, 1);
   BOOST_REQUIRE_CLOSE(gradient(0), target(0), 1e-3);
 }
 
+/*
+ * Implementation of the Hard Shrink activation function test. The function is
+ * implemented as Hard Shrink layer in the file hardshrink.hpp
+ *
+ * @param input Input data used for evaluating the Hard Shrink activation function.
+ * @param target Target data used to evaluate the Hard Shrink activation.
+ */
+void CheckHardShrinkActivationCorrect(const arma::colvec input,
+                                      const arma::colvec target)
+{
+  HardShrink<> hardshrink;
+
+  // Test the activation function using the entire vector as input.
+  arma::colvec activations;
+  hardshrink.Forward(input, activations);
+  for (size_t i = 0; i < activations.n_elem; i++)
+  {
+    BOOST_REQUIRE_CLOSE(activations.at(i), target.at(i), 1e-3);
+  }
+}
+
+/*
+ * Implementation of the HardShrink activation function derivative test.
+ * The derivative function is implemented as HardShrink layer in the file
+ * hardshrink.hpp
+ *
+ * @param input Input data used for evaluating the HardShrink activation function.
+ * @param target Target data used to evaluate the HardShrink activation.
+ */
+void CheckHardShrinkDerivativeCorrect(const arma::colvec input,
+                                      const arma::colvec target)
+{
+  HardShrink<> hardshrink;
+
+  // Test the calculation of the derivatives using the entire vector as input.
+  arma::colvec derivatives;
+
+  // This error vector will be set to 1 to get the derivatives.
+  arma::colvec error = arma::ones<arma::colvec>(input.n_elem);
+  hardshrink.Backward(input, error, derivatives);
+  for (size_t i = 0; i < derivatives.n_elem; i++)
+  {
+    BOOST_REQUIRE_CLOSE(derivatives.at(i), target.at(i), 1e-3);
+  }
+}
+
+/*
+ * Implementation of the Soft Shrink activation function test. The function is
+ * implemented as Soft Shrink layer in the file softshrink.hpp.
+ *
+ * @param input Input data used for evaluating the Soft Shrink activation function.
+ * @param target Target data used to evaluate the Soft Shrink activation.
+ */
+void CheckSoftShrinkActivationCorrect(const arma::colvec input,
+                                      const arma::colvec target)
+{
+  SoftShrink<> softshrink;
+
+  // Test the activation function using the entire vector as input.
+  arma::colvec activations;
+  softshrink.Forward(input, activations);
+  for (size_t i = 0; i < activations.n_elem; i++)
+  {
+    BOOST_REQUIRE_CLOSE(activations.at(i), target.at(i), 1e-3);
+  }
+}
+
+/*
+ * Implementation of the Soft Shrink activation function derivative test.
+ * The derivative function is implemented as Soft Shrink layer in the file
+ * softshrink.hpp
+ *
+ * @param input Input data used for evaluating the Soft Shrink activation function.
+ * @param target Target data used to evaluate the Soft Shrink activation.
+ */
+void CheckSoftShrinkDerivativeCorrect(const arma::colvec input,
+                                      const arma::colvec target)
+{
+  SoftShrink<> softshrink;
+
+  // Test the calculation of the derivatives using the entire vector as input.
+  arma::colvec derivatives;
+
+  // This error vector will be set to 1 to get the derivatives.
+  arma::colvec error = arma::ones<arma::colvec>(input.n_elem);
+  softshrink.Backward(input, error, derivatives);
+  for (size_t i = 0; i < derivatives.n_elem; i++)
+  {
+    BOOST_REQUIRE_CLOSE(derivatives.at(i), target.at(i), 1e-3);
+  }
+}
+
 /*
  * Simple SELU activation test to check whether the mean and variance remain
  * invariant after passing normalized inputs through the function.
@@ -342,7 +436,7 @@ BOOST_AUTO_TEST_CASE(SELUFunctionNormalizedTest)
 
   SELU selu;
 
-  selu.Forward(std::move(input), output);
+  selu.Forward(input, output);
 
   BOOST_REQUIRE_LE(arma::as_scalar(arma::abs(arma::mean(input) -
       arma::mean(output))), 0.1);
@@ -364,7 +458,7 @@ BOOST_AUTO_TEST_CASE(SELUFunctionUnnormalizedTest)
 
   SELU selu;
 
-  selu.Forward(std::move(input), output);
+  selu.Forward(input, output);
 
   BOOST_REQUIRE_GE(arma::as_scalar(arma::abs(arma::mean(input) -
       arma::mean(output))), 0.1);
@@ -388,23 +482,69 @@ BOOST_AUTO_TEST_CASE(SELUFunctionDerivativeTest)
 
   SELU selu;
 
-  selu.Forward(std::move(input), activations);
-  selu.Backward(std::move(activations), std::move(error),
-      std::move(derivatives));
+  selu.Forward(input, activations);
+  selu.Backward(activations, error, derivatives);
 
   BOOST_REQUIRE_LE(arma::as_scalar(arma::abs(arma::mean(derivatives) -
       selu.Lambda())), 10e-4);
 
   input.fill(-1);
 
-  selu.Forward(std::move(input), activations);
-  selu.Backward(std::move(activations), std::move(error),
-      std::move(derivatives));
+  selu.Forward(input, activations);
+  selu.Backward(activations, error, derivatives);
 
   BOOST_REQUIRE_LE(arma::as_scalar(arma::abs(arma::mean(derivatives) -
       selu.Lambda() * selu.Alpha() - arma::mean(activations))), 10e-4);
 }
 
+/**
+ * Implementation of the CELU activation function test. The function is
+ * implemented as CELU layer in the file celu.hpp.
+ *
+ * @param input Input data used for evaluating the CELU activation function.
+ * @param target Target data used to evaluate the CELU activation.
+ */
+void CheckCELUActivationCorrect(const arma::colvec input,
+                                const arma::colvec target)
+{
+  // Initialize CELU object with alpha = 1.0.
+  CELU<> lrf(1.0);
+
+  // Test the activation function using the entire vector as input.
+  arma::colvec activations;
+  lrf.Forward(input, activations);
+  for (size_t i = 0; i < activations.n_elem; i++)
+  {
+    BOOST_REQUIRE_CLOSE(activations.at(i), target.at(i), 1e-3);
+  }
+}
+
+/**
+ * Implementation of the CELU activation function derivative test. The function
+ * is implemented as CELU layer in the file celu.hpp.
+ *
+ * @param input Input data used for evaluating the CELU activation function.
+ * @param target Target data used to evaluate the CELU activation.
+ */
+void CheckCELUDerivativeCorrect(const arma::colvec input,
+                                const arma::colvec target)
+{
+  // Initialize CELU object with alpha = 1.0.
+  CELU<> lrf(1.0);
+
+  // Test the calculation of the derivatives using the entire vector as input.
+  arma::colvec derivatives, activations;
+
+  // This error vector will be set to 1 to get the derivatives.
+  arma::colvec error = arma::ones<arma::colvec>(input.n_elem);
+  lrf.Forward(input, activations);
+  lrf.Backward(activations, error, derivatives);
+  for (size_t i = 0; i < derivatives.n_elem; i++)
+  {
+    BOOST_REQUIRE_CLOSE(derivatives.at(i), target.at(i), 1e-3);
+  }
+}
+
 /**
  * Basic test of the tanh function.
  */
@@ -579,12 +719,11 @@ BOOST_AUTO_TEST_CASE(CReLUFunctionTest)
   CReLU<> crelu;
   // Test the activation function using the entire vector as input.
   arma::colvec activations;
-  crelu.Forward(std::move(activationData), std::move(activations));
+  crelu.Forward(activationData, activations);
   arma::colvec derivatives;
   // This error vector will be set to 1 to get the derivatives.
   arma::colvec error = arma::ones<arma::colvec>(desiredActivations.n_elem);
-  crelu.Backward(std::move(desiredActivations), std::move(error),
-        std::move(derivatives));
+  crelu.Backward(desiredActivations, error, derivatives);
   for (size_t i = 0; i < activations.n_elem; i++)
   {
     BOOST_REQUIRE_CLOSE(activations.at(i), desiredActivations.at(i), 1e-3);
@@ -634,4 +773,123 @@ BOOST_AUTO_TEST_CASE(HardSigmoidFunctionTest)
       desiredDerivatives);
 }
 
+/**
+ * Basic test of the Mish function.
+ */
+BOOST_AUTO_TEST_CASE(MishFunctionTest)
+{
+  // Calculated using tfa.activations.mish().
+  // where tfa is tensorflow_addons.
+  const arma::colvec desiredActivations("-0.25250152 3.1901977 \
+                                         4.498914 -3.05183208e-42 0.86509836 \
+                                         -0.30340138 1.943959 0");
+
+  const arma::colvec desiredDerivatives("0.4382387  1.0159768849 \
+                                          1.0019108 0.6 \
+                                          1.0192586  0.40639898 \
+                                          1.0725079  0.6");
+
+  CheckActivationCorrect<MishFunction>(activationData,
+                                       desiredActivations);
+  CheckDerivativeCorrect<MishFunction>(desiredActivations,
+                                        desiredDerivatives);
+}
+
+/**
+ * Basic test of the LiSHT function.
+ */
+BOOST_AUTO_TEST_CASE(LiSHTFunctionTest)
+{
+  // Calculated using tfa.activations.LiSHT().
+  // where tfa is tensorflow_addons.
+  const arma::colvec desiredActivations("1.928055 3.189384 \
+                                         4.4988894 100.2 0.7615942 \
+                                         0.7615942 1.9280552 0");
+
+  const arma::colvec desiredDerivatives("1.1150033 1.0181904 \
+                                         1.001978 1.0 \
+                                         1.0896928 1.0896928 \
+                                         1.1150033 0.0");
+
+  CheckActivationCorrect<LiSHTFunction>(activationData,
+                                        desiredActivations);
+  CheckDerivativeCorrect<LiSHTFunction>(desiredActivations,
+                                        desiredDerivatives);
+}
+
+/**
+ * Basic test of the GELU function.
+ */
+BOOST_AUTO_TEST_CASE(GELUFunctionTest)
+{
+  // Calculated using torch.nn.gelu().
+  const arma::colvec desiredActivations("-0.04540230591222 \
+                                          3.1981304348379158 \
+                                          4.5000 -0.0000 \
+                                          0.84119199060827676 \
+                                          -0.15880800939172329 \
+                                          1.954597694087775 \
+                                          0.0000");
+
+  const arma::colvec desiredDerivatives("0.46379920685377229 \
+                                          1.0065302165778773 \
+                                          1.0000293221871797 \
+                                          0.5 \
+                                          1.0351344625840642 \
+                                          0.37435387859861063 \
+                                          1.0909840032535403 \
+                                          0.5");
+
+  CheckActivationCorrect<GELUFunction>(activationData,
+                                       desiredActivations);
+  CheckDerivativeCorrect<GELUFunction>(desiredActivations,
+                                        desiredDerivatives);
+}
+
+/**
+ * Basic test of the Hard Shrink function.
+ */
+BOOST_AUTO_TEST_CASE(HardShrinkFunctionTest)
+{
+  const arma::colvec desiredActivations("-2 3.2 4.5 -100.2 1 -1 2 0");
+
+  const arma::colvec desiredDerivatives("1 1 1 1 1 1 1 0");
+
+  CheckHardShrinkActivationCorrect(activationData,
+                                   desiredActivations);
+  CheckHardShrinkDerivativeCorrect(desiredActivations,
+                                   desiredDerivatives);
+}
+
+/**
+ * Basic test of the Soft Shrink function.
+ */
+BOOST_AUTO_TEST_CASE(SoftShrinkFunctionTest)
+{
+  const arma::colvec desiredActivations("-1.5 2.7 4 -99.7 0.5 -0.5 1.5 0");
+
+  const arma::colvec desiredDerivatives("1 1 1 1 1 1 1 0");
+
+  CheckSoftShrinkActivationCorrect(activationData,
+                                   desiredActivations);
+  CheckSoftShrinkDerivativeCorrect(desiredActivations,
+                                   desiredDerivatives);
+}
+
+/**
+ * Basic test of the CELU activation function.
+ */
+BOOST_AUTO_TEST_CASE(CELUFunctionTest)
+{
+  const arma::colvec desiredActivations("-0.86466472 3.2 4.5 \
+                                         -1 1 -0.63212056 2 0");
+
+  const arma::colvec desiredDerivatives("0.42119275 1 1 \
+                                         0.36787944 1 \
+                                         0.5314636 1 1");
+
+  CheckCELUActivationCorrect(activationData, desiredActivations);
+  CheckCELUDerivativeCorrect(desiredActivations, desiredDerivatives);
+}
+
 BOOST_AUTO_TEST_SUITE_END();
